'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

_safeRequire('./jinghuan');

_safeRequire('./props');

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const path = _safeRequire('path');

const cluster = _safeRequire('cluster');

const pm2 = _safeRequire('./core/pm2');

const Watcher = _safeRequire('./core/watcher');

const Loaders = _safeRequire('./loaders');

const Cluster = _safeRequire('jinghuan-cluster');

const define = _safeRequire('jinghuan-helper/define');

const isArray = _safeRequire('lodash/isArray');

const fs = _safeRequire('fs-extra');

const isFunction = _safeRequire('lodash/isFunction');

let Application = class Application {
    constructor(options = {}) {
        this.watcherCallBack = fileInfo => {
            if (this.masterInstance) {
                this.masterInstance.forceReloadWorkers();
            } else {}
        };

        define('ROOT_PATH', options.ROOT_PATH, jinghuan);
        define('APP_PATH', options.APP_PATH, jinghuan);
        define('JH_PATH', options.JH_PATH, jinghuan);
        define('workers', options.workers, jinghuan);
        define('source', options.source, jinghuan);
        define('modules', options.modules.slice(), jinghuan);
        define('env', options.env, jinghuan);
        define('PORT', options.port || 8409, jinghuan);
        define('HOST', options.host, jinghuan);
        define('mode', options.mode, jinghuan);
        define('paths', options.paths, jinghuan);
        define('process_id', options.process_id, jinghuan);
        define('watcher', options.watcher, jinghuan);
        define('socket', options.socket, jinghuan);
    }

    notifier(err) {
        if (!jinghuan.notifier) {
            return;
        }
        let notifier = jinghuan.notifier;
        if (!isArray(notifier)) {
            notifier = [notifier];
        }
        notifier[0](Object.assign({
            title: 'JinghuanJs Transpile Error',
            message: err.message
        }, notifier[1]));
    }

    startWatcher() {
        if (!jinghuan.watcher) {
            return;
        }
        const srcPath = [path.join(jinghuan.ROOT_PATH, 'config', jinghuan.env)];

        let newModules = jinghuan.modules.concat('common');

        newModules.map(dirName => {
            let jhFile = path.join(jinghuan.ROOT_PATH, jinghuan.source, dirName, '.jinghuanjs');
            if (fs.pathExistsSync(jhFile)) {
                let conf = fs.readJsonSync(jhFile);
                for (let cpath of conf.paths || {}) {
                    srcPath.push(path.join(jinghuan.ROOT_PATH, jinghuan.source, dirName, cpath));
                }
            }
        });

        const instance = new Watcher({
            srcPath: srcPath
        }, fileInfo => {
            this.watcherCallBack(fileInfo);
        });

        instance.watch();
    }

    getMasterInstance() {
        this.masterInstance = new Cluster.Master({
            port: jinghuan.PORT,
            host: jinghuan.HOST,
            workers: jinghuan.workers
        });

        return this.masterInstance;
    }

    runInMaster(tag = "Master") {

        let lines = [];
        lines.push(`[${tag}] JinghuanJs version   ${jinghuan.version}`);
        lines.push(`[${tag}] HOST                 [${jinghuan.HOST}]`);
        lines.push(`[${tag}] PORT                 ${jinghuan.PORT}`);
        lines.push(`[${tag}] ROOT_PATH            ${jinghuan.ROOT_PATH}`);
        lines.push(`[${tag}] APP_PATH             ${jinghuan.APP_PATH}`);
        lines.push(`[${tag}] JH_PATH              ${jinghuan.JH_PATH}`);
        lines.push(`[${tag}] Enviroment           ${jinghuan.env}`);
        lines.push(`[${tag}] Source               ${jinghuan.source}`);
        lines.push(`[${tag}] Mode                 ${jinghuan.mode}`);
        lines.push(`[${tag}] Modules              [${jinghuan.modules}]`);
        lines.push(`[${tag}] Workers              ${jinghuan.workers}`);
        lines.push(`[${tag}] Watcher              ${jinghuan.watcher ? 'true' : 'false'}`);
        this.consoleLines(lines, '-');

        let instance = this.getMasterInstance();

        Promise.resolve(instance.startServer()).then(() => {
            jinghuan.logger.info(`[${tag}] Start OK`);

            let conf = jinghuan.config(tag.toLocaleLowerCase());
            if (isFunction(conf)) {
                conf();
            }
        });
    }

    getWorkerInstance() {
        let app = jinghuan.config('app') || {};

        this.workerInstance = new Cluster.Worker(_extends({}, app, {
            workers: jinghuan.workers,
            createServer: (() => {
                var _ref = _asyncToGenerator(function* () {
                    return jinghuan.app.listen(jinghuan.PORT, function () {});
                });

                return function createServer() {
                    return _ref.apply(this, arguments);
                };
            })()
        }));
        return this.workerInstance;
    }

    runInWorker(tag = "Worker") {

        let lines = [];
        lines.push(`[${tag}] JinghuanJs version   ${jinghuan.version}`);
        lines.push(`[${tag}] HOST                 [${jinghuan.HOST}]`);
        lines.push(`[${tag}] PORT                 ${jinghuan.PORT}`);
        lines.push(`[${tag}] ROOT_PATH            ${jinghuan.ROOT_PATH}`);
        lines.push(`[${tag}] APP_PATH             ${jinghuan.APP_PATH}`);
        lines.push(`[${tag}] JH_PATH              ${jinghuan.JH_PATH}`);
        lines.push(`[${tag}] Enviroment           ${jinghuan.env}`);
        lines.push(`[${tag}] Source               ${jinghuan.source}`);
        lines.push(`[${tag}] Mode                 ${jinghuan.mode}`);
        lines.push(`[${tag}] Modules              [${jinghuan.modules}]`);
        lines.push(`[${tag}] Middleware           [${jinghuan.middlewares}]`);
        lines.push(`[${tag}] Socket               [${jinghuan.sockets}]`);
        lines.push(`[${tag}] ID                   ${jinghuan.process_id}`);
        this.consoleLines(lines, '=');

        let instance = this.getWorkerInstance();

        Promise.resolve(instance.startServer()).then(() => {
            this.init = true;
            jinghuan.logger.info(`[${tag}] Start OK`);

            let conf = jinghuan.config(tag.toLocaleLowerCase());
            if (isFunction(conf)) {
                conf();
            }
        });
    }

    consoleLines(lines, flag = "*") {
        let length = 0;
        let index = 0;
        let flags = [];
        lines.forEach(str => {
            length = str.length > length ? str.length : length;
        });

        while (index++ < length) {
            flags.push(flag);
        }
        jinghuan.logger.info(flags.join(''));
        lines.forEach(str => {
            jinghuan.logger.info(str);
        });
        jinghuan.logger.info(flags.join(''));
    }

    initApp() {
        let Koa = _safeRequire('koa');
        if (jinghuan.socket) {
            let socket = _safeRequire('./core/socket');
            let app = socket(new Koa());
            define('app', app, jinghuan);
        } else {
            let app = new Koa();
            define('app', app, jinghuan);
        }
    }

    run() {
        if (pm2.isClusterMode) {
            throw new Error('can not use pm2 cluster mode, please change exec_mode to fork');
        }

        if (cluster.isMaster) {
            this.startWatcher();
        }

        const loaders = new Loaders(jinghuan);

        try {
            if (cluster.isMaster) {
                loaders.loadAll('master');
                return this.runInMaster();
            } else {
                this.initApp();
                loaders.loadAll('worker');
                return this.runInWorker();
            }
        } catch (e) {
            console.error(e);
        }
    }
};
;

exports.default = Application;

function _safeRequire(obj) {
    if (typeof obj === 'string') {
        try {
            obj = require(obj);
        } catch (e) {
            console.error(e);
            obj = null;
        }
    }

    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
}