"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

const path = _safeRequire("path");

const fs = _safeRequire("fs");

const assert = _safeRequire("assert");

const url = _safeRequire("url");

const onFinished = _safeRequire("on-finished");

const destroy = _safeRequire("destroy");

const events = _safeRequire("../core/events");

const get = _safeRequire("lodash/get");

const isObject = _safeRequire("lodash/isObject");

const ms = _safeRequire("jinghuan-helper/ms");

const PARAM = Symbol('context-param');
const POST = Symbol('context-post');
const FILE = Symbol('context-file');
const EVENTS = Symbol('context-events');
exports.default = {
    get userAgent() {
        return this.header['user-agent'];
    },

    get isGet() {
        return this.method === 'GET';
    },

    get isPost() {
        return this.method === 'POST';
    },

    get isSocket() {
        return this.websocket != null;
    },

    referer(onlyHost) {
        return this.referrer(onlyHost);
    },

    referrer(onlyHost) {
        const referrer = this.header['referer'];
        if (!referrer || !onlyHost) {
            return referrer;
        }
        return url.parse(referrer).hostname;
    },

    isMethod(method) {
        return this.method === method.toUpperCase();
    },

    isAjax(method) {
        if (method && !this.isMethod(method)) {
            return false;
        }
        return this.header['x-requested-with'] === 'XMLHttpRequest';
    },

    expires(time) {
        time = ms(time);
        const date = new Date(Date.now() + time);
        this.set('Cache-Control', `max-age=${time}`);
        this.set('Expires', date.toUTCString());
    },

    config(name, value) {
        return jinghuan.config(name, value);
    },

    param(name) {
        if (!this[PARAM]) {
            this[PARAM] = Object.assign({}, this.request._query || this.request.query);
        }
        if (!name) {
            return this[PARAM];
        }
        return get(this[PARAM], name, {});
    },

    post(name) {
        if (!this[POST]) {
            this[POST] = this.request.body && this.request.body.post;
        }
        if (!name) {
            return this[POST];
        }
        return get(this[POST], name, {});
    },

    file(name, value) {
        if (!this[FILE]) {
            this[FILE] = Object.assign({}, this.request.body && this.request.body.file);
        }
        if (!name) {
            return this[FILE];
        }
        if (isObject(name)) {
            this[FILE] = Object.assign(this[FILE], name);
            return this;
        }
        if (value === undefined) {
            return this[FILE][name];
        }
        this[FILE][name] = value;
        return this;
    },

    cookie(name, value, options = {}) {
        if (name == null && value == null) {
            return this.cookies.get();
        }
        if (value == null) {
            return this.cookies.get(name);
        }
        this.cookies.set(name, value, options);
    },

    download(filepath, filename = path.basename(filepath)) {
        assert(filepath, 'filepath can not be empty');
        const contentType = this.response.get('Content-Type');
        if (!contentType) {
            this.type = path.extname(filename);
        }
        const contentDisposition = this.response.get('Content-Disposition');
        if (!contentDisposition) {
            this.attachment(filename);
        }
        const stream = fs.createReadStream(filepath);
        this.body = stream;
        onFinished(this.res, () => {
            destroy(stream);
        });
        return false;
    },

    get events() {
        if (!this[EVENTS]) {
            this[EVENTS] = new events();
        }
        return this[EVENTS];
    },

    db(...args) {
        return jinghuan.db(...args);
    },

    session(name, value, option) {},

    cache(...args) {
        return jinghuan.cache(...args);
    }
};

function _safeRequire(obj) {
    if (typeof obj === 'string') {
        try {
            obj = require(obj);
        } catch (e) {
            console.error(e);
            obj = null;
        }
    }

    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
}