'use strict';

const bluebird = _safeRequire('bluebird');

const logger = _safeRequire('jinghuan-logger');

const pkg = _safeRequire('../package.json');

const helper = _safeRequire('jinghuan-helper');

const events = _safeRequire('./core/events');

const define = _safeRequire('jinghuan-helper/define');

const JinghuanDb = _safeRequire('jinghuan-db');

const JinghuanCache = _safeRequire('jinghuan-cache');

const JinghuanValidator = _safeRequire('jinghuan-validator');

const get = _safeRequire('lodash/get');

const isObject = _safeRequire('lodash/isObject');

const db = JinghuanDb(function (name) {
  return jinghuan.config(`database.${name}`);
});

const cache = JinghuanCache(function (name) {
  return jinghuan.config(`cache.${name}`);
});

const validator = JinghuanValidator(function (name) {
  return jinghuan.config(`validate.${name}`);
});

const _get = function (name, ctx, def = null) {
  if (isObject(jinghuan[name])) {
    let { module, controller, action } = ctx;
    let ret = null;
    if (!ret) {
      ret = get(jinghuan[name], `${module}.${controller}/${action}`);
    }
    if (!ret) {
      ret = get(jinghuan[name], `${module}.${controller}.${action}`);
    }
    if (!ret) {
      ret = get(jinghuan[name], `${module}.${controller}`);
    }
    return ret || def;
  }
  return def;
};

let Controller = class Controller {};

define('Promise', bluebird, global);

define('jinghuan', Object.create(helper), global);

define('version', pkg.version, jinghuan);

define('validator', validator, jinghuan);

define('Controller', Controller, jinghuan);

define('logger', logger, jinghuan);

define('events', new events(), jinghuan);

define('db', db, jinghuan);

define('cache', cache, jinghuan);

define('get', _get, jinghuan);

function _safeRequire(obj) {
  if (typeof obj === 'string') {
    try {
      obj = require(obj);
    } catch (e) {
      console.error(e);
      obj = null;
    }
  }

  return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
}