'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

const path = _safeRequire('path');

const Config = _safeRequire('jinghuan-config');

const Loader = _safeRequire('./core/loader');

const define = _safeRequire('jinghuan-helper/define');

const cluster = _safeRequire('cluster');

let Loaders = class Loaders {
    constructor(options = {}) {
        this.options = options;
    }

    loadData() {
        let events = this.loader.loadEvents();
        define('controllers', this.loader.loadController(), jinghuan);
        define('sql', this.loader.loadSql(), jinghuan);
        define('validate', this.loader.loadValidate(), jinghuan);
    }

    loadMiddleware() {
        const middlewares = this.loader.loadMiddleware();
        middlewares.forEach(middleware => {
            jinghuan.app.use(middleware);
        });
    }

    loadSocket() {
        const sockets = this.loader.loadSocket();
        sockets.forEach(socket => {
            jinghuan.app.ws.use(socket);
        });
    }

    loadExtend() {

        let {
            JH_PATH,
            ROOT_PATH
        } = jinghuan;

        let exts = this.loader.loadExtend(path.join(JH_PATH));

        let list = [['jinghuan', jinghuan], ['application', jinghuan.app], ['context', jinghuan.app.context], ['controller', jinghuan.Controller.prototype]];

        list.forEach(item => {
            if (!exts[item[0]]) {
                return;
            }
            Loader.extend(item[1], exts[item[0]]);
        });

        exts = this.loader.loadExtend(path.join(ROOT_PATH, jinghuan.source, 'common'));

        list = [['jinghuan', jinghuan], ['application', jinghuan.app], ['context', jinghuan.app.context], ['controller', jinghuan.Controller.prototype]];

        list.forEach(item => {
            if (!exts[item[0]]) {
                return;
            }
            Loader.extend(item[1], exts[item[0]]);
        });
    }

    loadConfig() {
        let { ROOT_PATH, JH_PATH, env, source } = jinghuan;

        const jinghuanConfig = path.join(JH_PATH, 'config');

        const commonConfig = path.join(ROOT_PATH, source, '/common/config');

        const envConfig = path.join(ROOT_PATH, 'config', env);

        define('config', Config([jinghuanConfig, commonConfig, envConfig]), jinghuan);
    }

    loadAll(type) {

        this.loader = new Loader();

        this.loadConfig();

        if (type !== 'master') {
            this.loadExtend();
            this.loadData();
            this.loadMiddleware();
            this.loadSocket();
        }
    }
};
;

exports.default = Loaders;

function _safeRequire(obj) {
    if (typeof obj === 'string') {
        try {
            obj = require(obj);
        } catch (e) {
            console.error(e);
            obj = null;
        }
    }

    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
}