'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

const getController = _safeRequire('jinghuan-helper/getController');

const get = _safeRequire('lodash/get');

const symbol = PropAction.name;
const defaultOptions = {};

function MidController(options, app) {

    options = Object.assign({}, defaultOptions, options);

    return (ctx, next) => {

        if (!ctx.module || !ctx.controller || !ctx.action) {
            ctx.status = 404;
            return;
        }

        let instance = getController(ctx, jinghuan.controllers, symbol);

        if (!instance) {
            ctx.status = 404;
            return;
        }

        let param = ctx.param() || {};
        let post = ctx.post() || {};
        let file = ctx.file() || {};
        let actions = instance[symbol];
        let promise = Promise.resolve();

        if (instance.__before) {
            promise = Promise.resolve(instance.__before(param, post, file));
        }

        return promise.then(data => {
            if (data === false) {
                return false;
            }
            let action = ctx.action;
            if (actions[action]) {
                if (actions[action].value) {
                    return actions[action].value.call(instance, param, post, file);
                } else if (actions[action].initializer) {
                    return actions[action].initializer.call(instance)(param, post, file);
                }
            }
        }).then(data => {
            if (data === false) {
                return false;
            }
            if (instance.__after) {
                return instance.__after(param, post, file);
            }
        }).then(data => {
            if (data !== false) {
                return next();
            }
        }).catch(e => {
            ctx.status = 500;
            console.error(e);
        });
    };
}

exports.default = MidController;

function _safeRequire(obj) {
    if (typeof obj === 'string') {
        try {
            obj = require(obj);
        } catch (e) {
            console.error(e);
            obj = null;
        }
    }

    return obj && obj.__esModule && typeof obj.default !== "undefined" ? obj.default : obj;
}